/**
 * Explorer block behavior for Drupal 11.
 */
(function (Drupal, $, once) {
  'use strict';

  /**
   * Constants for supported file extensions.
   */
  const SUPPORTED_EXTENSIONS = {
    // Extensions that can be viewed in Google Doc Viewer
    GOOGLE_DOC: ['doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx'],

    // Extensions that can be opened directly in browser
    OPEN_DIRECT: [
      'txt', 'pdf', 'docx', 'xlsx', 'pptx', 'odt', 'ods', 'odp', 'csv',
      'jpg', 'jpeg', 'png', 'gif', 'bmp', 'svg',
      'mp3', 'wav', 'aac', 'ogg',
      'mp4', 'webm', 'ogv', 'mov',
    ]
  };

  /**
   * URL constants.
   */
  const GOOGLE_VIEWER_URL = 'https://docs.google.com/gview?embedded=true&url=';

  /**
   * Explorer block behavior implementation.
   */
  Drupal.behaviors.explorerblock = {
    attach: function (context, settings) {
      once('browser_block', '.browser-block', context).forEach(element => {
        const $block = $(element);
        const config = {
          urlScan: Drupal.checkPlain($block.data('url_scan')),
          url: Drupal.checkPlain($block.data('url')),
          all: $block.data('all'),
          rootFolder: decodeURI($block.data('folder').replace('\\', '/')),
          googleViewer: $block.data('google_viewer')
        };
        this.initBrowser($block, config);
      });
    },

    /**
     * Initialize browser with configuration.
     *
     * @param {jQuery} $element - The block element.
     * @param {Object} config - Configuration options.
     */
    initBrowser: function($element, config) {
      $.post(config.urlScan, {directory: config.rootFolder, all: config.all})
        .then(environmentData => {
          const fileSystem = new FileSystemBrowser(environmentData, config);
          const browse = {
            root: '/',
            separator: '/',
            contextmenu: true,
            name: 'filestystem',
            refresh_timer: 0,
            dir: path => fileSystem.getDirectoryContents(path),
            open: filename => fileSystem.openFile(filename, config.urlScan)
          };
          $element.browse(browse);
        })
        .catch(error => {
          console.error('Failed to load directory structure:', error);
        });
    }
  };

  /**
   * Class to handle file system browsing operations.
   */
  class FileSystemBrowser {
    /**
     * @param {Object} environment - The initial environment data.
     * @param {Object} config - Configuration options.
     */
    constructor(environment, config) {
      this.environment = environment;
      this.config = config;
      this.currentNode = {};
    }

    /**
     * Navigate to a specific path in the file system.
     *
     * @param {string} path - The path to navigate to.
     * @returns {Object} The current node at that path.
     */
    navigateTo(path) {
      this.currentNode = this.environment;

      if (path && path !== '/') {
        this.walkPath(path, segment => {
          if (this.currentNode[segment] !== undefined) {
            this.currentNode = this.currentNode[segment];
          }
        });
      }

      return this.currentNode;
    }

    /**
     * Walk through a path executing a callback for each segment.
     *
     * @param {string} path - The path to walk through.
     * @param {Function} callback - Function to execute for each segment.
     */
    walkPath(path, callback) {
      const segments = path.split('/').filter(Boolean);
      segments.forEach(callback);
    }

    /**
     * Get directory contents for a specific path.
     *
     * @param {string} path - The path to get contents for.
     * @returns {Promise<Object>} Directory listing with files and dirs.
     */
    async getDirectoryContents(path) {
      const directory = this.navigateTo(path);
      const result = { files: [], dirs: [] };

      if (!$.isPlainObject(directory)) {
        return Promise.resolve(result);
      }
      const directoryTasks = Object.keys(directory).map(async key => {
        if (typeof directory[key] === 'string') {
          result.files.push(key);
        } else {
          result.dirs.push(key);

          // Load subdirectory contents if empty
          if (Object.keys(directory[key]).length === 0) {
            try {
              const subDirPath = this.config.rootFolder + path + '/' + key;
              const data = await $.post(this.config.urlScan, {
                directory: subDirPath,
                root: this.config.rootFolder
              });
              directory[key] = data;
            } catch (error) {
              console.error(`Failed to load subdirectory ${key}:`, error);
            }
          }
        }
      });
      await Promise.all(directoryTasks);
      return result;
    }

    /**
     * Open a file at the specified path.
     *
     * @param {string} filename - The path to the file.
     */
    openFile(filename) {
      const file = this.navigateTo(filename);

      if (typeof file !== 'string') {
        return;
      }

      const filePath = file.split('web/').pop();
      const fileUrl = this.config.url + '/' + filePath.replace('//','/');
      const extension = fileUrl.split('.').pop().toLowerCase();

      this.handleFileOpen(fileUrl, extension, filename);
    }

    /**
     * Handle file opening based on file type.
     *
     * @param {string} fileUrl - The URL to the file.
     * @param {string} extension - The file extension.
     * @param {string} filename - The original filename path.
     */
    handleFileOpen(fileUrl, extension, filename) {
      const link = document.createElement('a');

      if (SUPPORTED_EXTENSIONS.OPEN_DIRECT.includes(extension)) {
        // Open directly in browser
        link.href = fileUrl;
        link.target = '_blank';
      } else if (this.config.googleViewer && SUPPORTED_EXTENSIONS.GOOGLE_DOC.includes(extension)) {
        // Open in Google Viewer
        link.href = GOOGLE_VIEWER_URL + fileUrl;
        link.target = '_blank';
      } else {
        // Download the file
        link.href = fileUrl;
        link.download = filename.split('/').pop();
      }

      // Use modern approach for clicking the link
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
    }
  }

})(Drupal, jQuery, once);
